/***************************************************************************
 *   Copyright (C) 2009-2010 by Daniel Nicoletti                           *
 *   dantti85-pk@yahoo.com.br                                              *
 *   Copyright (C) 2011 Kevin Kofler <kevin.kofler@chello.at>              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; see the file COPYING. If not, write to       *
 *   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,  *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#include "PkInstallPlasmaResources.h"

#include <KpkReviewChanges.h>
#include <KpkTransaction.h>
#include <KpkStrings.h>

#include <KLocale>
#include <KMessageBox>

#include <KDebug>

PkInstallPlasmaResources::PkInstallPlasmaResources(uint xid,
                                                   const QStringList &resources,
                                                   const QString &interaction,
                                                   const QDBusMessage &message,
                                                   QWidget *parent)
 : KpkAbstractTask(xid, interaction, message, parent),
   m_resources(resources)
{
}

PkInstallPlasmaResources::~PkInstallPlasmaResources()
{
}

void PkInstallPlasmaResources::start()
{
    int ret = KMessageBox::Yes;
    QStringList niceNames;
    QStringList search;
    if (showConfirmSearch()) {
        // Resources are strings like "dataengine-weather"
        foreach (const QString &service, m_resources) {
            QString prettyService = service;
            if (service.startsWith("dataengine-")) {
                prettyService = i18n("%1 data engine", service.mid(11));
            } else if (service.startsWith("scriptengine-")) {
                prettyService = i18n("%1 script engine", service.mid(13));
            }

            niceNames << prettyService;
            search << service;
        }

        QString message = i18np("The following service is required: <ul><li>%2</li></ul>"
                                "Do you want to search for this now?",
                                "The following services are required: <ul><li>%2</li></ul>"
                                "Do you want to search for these now?",
                                niceNames.size(),
                                niceNames.join("</li><li>"));

        QString title = i18np("Plasma requires an additional service for this operation",
                              "Plasma requires additional services for this operation",
                              m_resources.size());

        QString msg = "<h3>" + title + "</h3>" + message;
        KGuiItem searchBt = KStandardGuiItem::yes();
        searchBt.setText(i18nc("Search for packages" ,"Search"));
        searchBt.setIcon(KIcon("edit-find"));
        ret = KMessageBox::questionYesNoWId(parentWId(), msg, title, searchBt);
    }

    if (ret == KMessageBox::Yes) {
        Transaction *t = Client::instance()->whatProvides(Enum::ProvidesPlasmaService,
                                                          search,
                                                          Enum::FilterNotInstalled |
                                                          Enum::FilterArch |
                                                          Enum::FilterNewest);
        if (t->error()) {
            QString msg(i18n("Failed to search for provides"));
            KMessageBox::sorryWId(parentWId(), KpkStrings::daemonError(t->error()), msg);
            sendErrorFinished(InternalError, msg);
        } else {
            connect(t, SIGNAL(finished(PackageKit::Enum::Exit, uint)),
                    this, SLOT(whatProvidesFinished(PackageKit::Enum::Exit)));
            connect(t, SIGNAL(package(QSharedPointer<PackageKit::Package>)),
                    this, SLOT(addPackage(QSharedPointer<PackageKit::Package>)));
            if (showProgress()) {
                kTransaction()->setTransaction(t);
                kTransaction()->show();
            }
        }
    } else {
        sendErrorFinished(Cancelled, i18n("did not agree to search"));
    }
}

void PkInstallPlasmaResources::whatProvidesFinished(PackageKit::Enum::Exit status)
{
    kDebug() << "Finished.";
    if (status == Enum::ExitSuccess) {
        if (m_foundPackages.size()) {
            kTransaction()->hide();
            KpkReviewChanges *frm = new KpkReviewChanges(m_foundPackages, this, parentWId());
            if (frm->exec(operationModes()) == 0) {
                sendErrorFinished(Failed, "Transaction did not finish with success");
            } else {
                finishTaskOk();
            }
        } else {
            if (showWarning()) {
                KMessageBox::sorryWId(parentWId(),
                                      i18n("Could not find service "
                                           "in any configured software source"),
                                      i18n("Failed to search for Plasma service"));
            }
            sendErrorFinished(NoPackagesFound, "failed to find Plasma service");
        }
    } else {
        sendErrorFinished(Failed, "what provides failed");
    }
}

void PkInstallPlasmaResources::addPackage(QSharedPointer<PackageKit::Package> package)
{
    m_foundPackages.append(package);
}

#include "PkInstallPlasmaResources.moc"
